@@ -17,13 +17,15 @@
 
 """Committing to Subversion repositories."""
 
+from __future__ import absolute_import
+
 from collections import defaultdict
 
 from cStringIO import (
     StringIO,
     )
-import subvertpy
 from subvertpy import (
+    ERR_BAD_PROPERTY_VALUE ,
     ERR_FS_NOT_DIRECTORY,
     ERR_REPOS_DISABLED_FEATURE,
     delta,
@@ -80,8 +82,6 @@ from bzrlib.plugins.svn.util import (
 
 DUMMY_ROOT_PROPERTY_NAME = "bzr:svn:dummy"
 
-ERR_BAD_PROPERTY_VALUE = getattr(subvertpy, "ERR_BAD_PROPERTY_VALUE", 125005)
-
 _fileprops_warned_repos = set()
 
 def warn_setting_fileprops(uuid):
@@ -405,8 +405,8 @@ class SvnCommitBuilder(CommitBuilder):
 
     updates_branch = True
 
-    def __init__(self, repository, branch_path, parents, config, timestamp,
-                 timezone, committer, revprops, revision_id,
+    def __init__(self, repository, branch_path, parents, config_stack,
+                 timestamp, timezone, committer, revprops, revision_id,
                  base_foreign_revid, base_mapping, root_action, old_tree=None,
                  push_metadata=True, graph=None, opt_signature=None,
                  testament=None, branch=None, _rich_root_bump=False):
@@ -415,7 +415,7 @@ class SvnCommitBuilder(CommitBuilder):
         :param repository: SvnRepository to commit to.
         :param branch: branch path to commit to.
         :param parents: List of parent revision ids.
-        :param config: Branch configuration to use.
+        :param config_stack: Branch configuration stack to use.
         :param timestamp: Optional timestamp recorded for commit.
         :param timezone: Optional timezone for timestamp.
         :param committer: Optional committer to set for commit.
@@ -432,7 +432,7 @@ class SvnCommitBuilder(CommitBuilder):
         """
         self.push_metadata = push_metadata
         super(SvnCommitBuilder, self).__init__(repository, parents,
-            config, timestamp, timezone, committer, revprops, revision_id)
+            config_stack, timestamp, timezone, committer, revprops, revision_id)
         self._basis_delta = []
         self.new_inventory = None
         self._any_changes = False
@@ -513,7 +513,7 @@ class SvnCommitBuilder(CommitBuilder):
         if push_metadata:
             (self.set_custom_revprops,
                 self.set_custom_fileprops) = self.repository._properties_to_set(
-                    self.mapping, config)
+                    self.mapping, config_stack)
         else:
             self.set_custom_revprops = False
             self.set_custom_fileprops = False
@@ -776,7 +776,7 @@ class SvnCommitBuilder(CommitBuilder):
                     self._override_file_ids, self._svnprops)
                 self.mapping.export_text_parents_fileprops(
                     self._override_text_parents, self._svnprops)
-        if self._config.get_log_strip_trailing_newline():
+        if self._config_stack.get("log-strip-trailing-newline"):
             if self.set_custom_revprops:
                 self.mapping.export_message_revprops(message,
                     self._svn_revprops)
@@ -863,11 +863,10 @@ class SvnCommitBuilder(CommitBuilder):
                 finally:
                     for dir_editor in reversed(branch_editors):
                         dir_editor.close()
+                editor.close()
             except:
                 editor.abort()
                 raise
-            else:
-                editor.close()
         finally:
             self.repository.transport.add_connection(self.conn)
             self.conn = None
@@ -884,7 +883,7 @@ class SvnCommitBuilder(CommitBuilder):
         self.mutter('commit %d finished. author: %r, date: %r',
                result_revision, result_author, result_date)
 
-        override_svn_revprops = self._config.get_override_svn_revprops()
+        override_svn_revprops = self._config_stack.get('override-svn-revprops')
         if override_svn_revprops is not None:
             new_revprops = {}
             if (("%s=committer" % properties.PROP_REVISION_AUTHOR) in override_svn_revprops or
@@ -948,13 +947,13 @@ class SvnCommitBuilder(CommitBuilder):
         inv = mutable_inventory_from_tree(self.old_tree)
         revid = self._get_actual_revision_id()
         inv.apply_delta(self._get_basis_delta(revid))
-        return InventoryRevisionTree(self.repository, inv,
-            revid)
+        return InventoryRevisionTree(self.repository, inv, revid)
 
     def abort(self):
         if self.conn is not None:
             self.repository.transport.add_connection(self.conn)
-        self.repository.abort_write_group()
+        if self.repository.is_in_write_group():
+            self.repository.abort_write_group()
 
     def _visit_parent_dirs(self, path):
         """Add the parents of path to the list of paths to visit."""
