@@ -17,17 +17,19 @@
 
 """Stores per-repository settings."""
 
+from __future__ import absolute_import
 
 import os
 from subvertpy import (
     properties,
     )
 
-import bzrlib.config
 from bzrlib import (
     atomicfile,
+    config as _mod_bzr_config,
     osutils,
     trace,
+    transport,
     )
 from bzrlib.config import (
     ConfigObj,
@@ -44,8 +46,6 @@ from bzrlib.errors import (
     BzrError,
     )
 
-from bzrlib.config import LockableConfig
-
 
 def as_bool(str):
     """Parse a string as a boolean.
@@ -71,8 +71,68 @@ def as_bool(str):
 # was seen at.
 
 
+class SubversionStore(_mod_bzr_config.LockableIniFileStore):
 
-class SubversionUUIDConfig(LockableConfig):
+    def __init__(self, possible_transports=None):
+        t = transport.get_transport_from_path(
+            _mod_bzr_config.config_dir(),
+            possible_transports=possible_transports)
+        super(SubversionStore, self).__init__(t, 'subversion.conf')
+        self.id = 'subversion'
+
+
+class UUIDMatcher(_mod_bzr_config.SectionMatcher):
+    """UUID-based Subversion section matcher."""
+
+    def __init__(self, store, uuid):
+        super(UUIDMatcher, self).__init__(store)
+        self.uuid = uuid
+
+    def match(self, section):
+        return section.id == self.uuid
+
+
+class SvnBranchStack(_mod_bzr_config._CompatibleStack):
+    """SvnBranch stack providing UUID specific options."""
+
+    def __init__(self, branch_base, repository_uuid):
+        lstore = _mod_bzr_config.LocationStore()
+        loc_matcher = _mod_bzr_config.LocationMatcher(lstore, branch_base)
+        svn_store = SubversionStore()
+        uuid_matcher = UUIDMatcher(svn_store, repository_uuid)
+        gstore = _mod_bzr_config.GlobalStore()
+        super(SvnBranchStack, self).__init__(
+            [self._get_overrides,
+             loc_matcher.get_sections,
+             uuid_matcher.get_sections,
+             gstore.get_sections],
+            # All modifications go to the corresponding section in
+            # locations.conf
+            lstore, branch_base)
+
+
+class SvnRepositoryStack(_mod_bzr_config._CompatibleStack):
+    """SvnRepository stack providing UUID specific options."""
+
+    def __init__(self, repository):
+        lstore = _mod_bzr_config.LocationStore()
+        loc_matcher = _mod_bzr_config.LocationMatcher(lstore, repository.base)
+        svn_store = SubversionStore()
+        uuid_matcher = UUIDMatcher(svn_store,
+                                   getattr(repository, 'uuid', None))
+        gstore = _mod_bzr_config.GlobalStore()
+        super(SvnRepositoryStack, self).__init__(
+            [self._get_overrides,
+             loc_matcher.get_sections,
+             uuid_matcher.get_sections,
+             gstore.get_sections],
+            # All modifications go to the corresponding section in
+            # locations.conf
+            lstore, repository.base)
+        self.repository = repository
+
+
+class SubversionUUIDConfig(_mod_bzr_config.LockableConfig):
     """UUID-based Subversion configuration."""
 
     def __init__(self, uuid):
@@ -178,36 +238,6 @@ class SvnRepositoryConfig(Config):
     def set_user_option(self, name, value):
         self._get_uuid_config().set_user_option(name, value)
 
-    def get_layout(self):
-        return self._get_user_option("layout", use_global=False)
-
-    def get_guessed_layout(self):
-        return self._get_user_option("guessed-layout", use_global=False)
-
-    def set_layout(self, layout):
-        return self.set_user_option("layout", str(layout))
-
-    def set_guessed_layout(self, layout):
-        return self.set_user_option("guessed-layout", str(layout))
-
-    def get_branches(self):
-        branches_str = self._get_user_option("branches", use_global=False)
-        if branches_str is None:
-            return None
-        return [b.encode("utf-8") for b in branches_str.split(";") if b != ""]
-
-    def get_tags(self):
-        tags_str = self._get_user_option("tags", use_global=False)
-        if tags_str is None:
-            return None
-        return [t.encode("utf-8") for t in tags_str.split(";") if t != ""]
-
-    def set_tags(self, tags):
-        self.set_user_option("tags", "".join(["%s;" % t.decode("utf-8") for t in tags]))
-
-    def set_branches(self, branches):
-        self.set_user_option("branches", "".join(["%s;" % b.decode("utf-8") for b in branches]))
-
     def get_reuse_revisions(self):
         ret = self._get_user_option("reuse-revisions")
         if ret is None:
@@ -326,14 +356,6 @@ class BranchConfig(Config):
             self._global_config = GlobalConfig()
         return self._global_config
 
-    def get_log_strip_trailing_newline(self):
-        """Check whether or not trailing newlines should be stripped in the
-        Subversion log message (where support by the bzr<->svn mapping used)."""
-        try:
-            return self.get_bool("log-strip-trailing-newline")
-        except KeyError:
-            return False
-
     def get_bool(self, option_name, use_global=True):
         """See Config.get_bool."""
         ret = self._get_user_option(option_name, use_global)
@@ -341,21 +363,6 @@ class BranchConfig(Config):
             raise KeyError
         return as_bool(ret)
 
-    def get_override_svn_revprops(self):
-        """Check whether or not bzr-svn should attempt to override Subversion revision
-        properties after committing."""
-        try:
-            if self.get_bool("override-svn-revprops"):
-                return [properties.PROP_REVISION_DATE, properties.PROP_REVISION_AUTHOR]
-            return []
-        except ValueError:
-            val = self._get_user_option("override-svn-revprops")
-            if not isinstance(val, list):
-                return [val]
-            return val
-        except KeyError:
-            return None
-
     def _get_user_option(self, option_name, use_global=True):
         """See Config._get_user_option."""
         for source in self.option_sources:
@@ -366,23 +373,6 @@ class BranchConfig(Config):
                 return value
         return None
 
-    def get_allow_metadata_in_fileprops(self):
-        """Check whether file properties may be used to store bzr-svn metadata.
-
-        """
-        try:
-            return self.get_bool("allow_metadata_in_file_properties")
-        except KeyError:
-            return False
-
-    def get_append_revisions_only(self, default=True):
-        """Check whether it is possible to remove revisions from the mainline.
-        """
-        try:
-            return self.get_bool("append_revisions_only")
-        except KeyError:
-            return default
-
     def _get_user_id(self):
         """Get the user id from the 'email' key in the current section."""
         return self._get_user_option('email')
@@ -413,13 +403,6 @@ class BranchConfig(Config):
                         trace.warning('Value "%s" is masked by "%s" from'
                                       ' branch.conf', value, mask_value)
 
-    def get_push_merged_revisions(self):
-        """Check whether merged revisions should be pushed."""
-        try:
-            return self.get_bool("push_merged_revisions")
-        except KeyError:
-            return None
-
     def has_explicit_nickname(self):
         return False
 
@@ -493,3 +476,99 @@ class SubversionBuildPackageConfig(object):
 
     def __setitem__(self, option_name, value):
         self.option_source[option_name] = value
+
+
+svn_layout_option = _mod_bzr_config.Option('layout', default=None,
+           help='''\
+The mandated bzr-svn repository layout to use.
+
+See `bzr help svn-layout` for details.
+''')
+
+svn_guessed_layout_option = _mod_bzr_config.Option('guessed-layout',
+    default=None,
+    help='''\
+The bzr-svn repository layout that bzr-svn guessed.
+
+See `bzr help svn-layout` for details.
+''')
+
+def svn_paths_from_store(text):
+    return [b.encode("utf-8") for b in text.split(";") if b != ""]
+
+svn_branches_option = _mod_bzr_config.Option('branches', default=None,
+           from_unicode=svn_paths_from_store,
+           help='''\
+Paths in the Subversion repository to consider branches.
+
+This should be a comma-separated list of paths, each relative to the
+repository root. Asterisks may be used as wildcards, and will match
+a single path element.
+
+See `bzr help svn-layout` for more information on Subversion repository layouts.
+''')
+
+svn_tags_option = _mod_bzr_config.Option('tags', default=None,
+           from_unicode=svn_paths_from_store,
+           help='''\
+Paths in the Subversion repository to consider tags.
+
+This should be a comma-separated list of paths, each relative to the
+repository root. Asterisks may be used as wildcards, and will match
+a single path element.
+
+See `bzr help svn-layout` for more information on Subversion repository layouts.
+''')
+
+
+def override_svn_revprops_from_store(text):
+    val = _mod_bzr_config.bool_from_store(text)
+    if val is not None:
+        if val:
+            return [properties.PROP_REVISION_DATE, properties.PROP_REVISION_AUTHOR]
+        else:
+            return []
+    else:
+        # FIXME: Check for allowed values ?
+        return text.split(",")
+
+
+svn_override_revprops = _mod_bzr_config.Option('override-svn-revprops', default=None,
+    from_unicode=override_svn_revprops_from_store,
+   help='''\
+Which Subversion revision properties to override (comma-separated list).
+
+This is done after a revision is pushed from bzr, and requires the
+pre-revprop-change-hook to be enabled in the repository.
+
+Possible values:
+
+ * svn:author=committer - set the svn author to the committer of the bzr
+                          revision
+ * svn:author=author, svn:author - set the svn author to the first author of
+                                   the bzr revision
+ * svn:date - override the date to be the same as the commit timestamp of
+              the bzr revision
+''')
+
+svn_log_strip_trailing_new_line = _mod_bzr_config.Option('log-strip-trailing-newline',
+    default=False, from_unicode=_mod_bzr_config.bool_from_store,
+    help='''\
+Whether or not trailing newlines should be stripped in the Subversion log message.
+
+This only applies to revisions created by bzr-svn.
+''')
+svn_push_merged_revisions = _mod_bzr_config.Option('push_merged_revisions',
+    default=False, from_unicode=_mod_bzr_config.bool_from_store,
+    help='''\
+Whether to push merged revisions.
+''')
+svn_allow_metadata_in_fileprops = _mod_bzr_config.Option('allow_metadata_in_file_properties',
+    default=False, from_unicode=_mod_bzr_config.bool_from_store,
+    help='''\
+Whether to allow bzr-svn to store bzr-svn-specific metadata in Subversion file properties.
+
+This is usually a bad idea, and not necessary with newer versions of Subversion,
+which support revision properties. This setting is present for compatibility with
+older versions of bzr-svn.
+''')
