@@ -17,6 +17,8 @@
 
 """Generation of file-ids."""
 
+from __future__ import absolute_import
+
 import urllib
 
 from bzrlib import (
@@ -94,6 +96,7 @@ def idmap_reverse_lookup(idmap, mapping, fileid):
     :param fileid: The file id to look up
     :return: Path
     """
+    assert type(fileid) is str
     if mapping.parseable_file_ids:
         try:
             (uuid, revnum, path) = mapping.parse_file_id(fileid)
@@ -225,6 +228,7 @@ class DictFileIdMap(FileIdMap):
 
         for x in sorted(text_revisions.keys() + delta.keys()):
             assert isinstance(x, unicode)
+            # Entry was added
             if (x in delta and
                 (not x in self.data or self.data[x][0] != delta[x])):
                 if x in changes and changes[x][1] is not None:
@@ -236,6 +240,7 @@ class DictFileIdMap(FileIdMap):
                 self.data[x] = (delta[x],
                     text_revisions.get(x, default_revid), child_create_revid)
             else:
+                # Entry already existed
                 try:
                     prev_entry = self.lookup(mapping, x)
                 except KeyError:
@@ -294,6 +299,10 @@ class FileIdMapStore(object):
     def update_idmap(self, map, revmeta, mapping):
         local_changes = get_local_changes(revmeta.metarev.paths,
                 revmeta.metarev.branch_path)
+        parentrevmeta = revmeta.get_lhs_parent_revmeta(mapping)
+        if revmeta.get_lhs_parent_revid(mapping, parentrevmeta) == NULL_REVISION:
+            # Special case - a NULL_REVISION officially doesn't have a root
+            local_changes[u''] = ('A', None)
         idmap = self.get_idmap_delta(local_changes, revmeta, mapping)
         revid = revmeta.get_revision_id(mapping)
         text_revisions = determine_text_revisions(local_changes, revid,
@@ -306,7 +315,7 @@ class FileIdMapStore(object):
         (uuid, branch, revnum) = foreign_revid
         todo = []
         next_parent_revs = []
-        if mapping.is_branch(""):
+        if mapping.is_branch("") and branch == "":
             map = DictFileIdMap({
                 u"": (mapping.generate_file_id((uuid, "", 0), u""),
                       mapping.revision_id_foreign_to_bzr((uuid, "", 0)),
@@ -320,12 +329,12 @@ class FileIdMapStore(object):
             return map
 
         # No history -> empty map
-        todo = list(self.repos._iter_reverse_revmeta_mapping_history(branch, revnum, to_revnum=0, mapping=mapping))
+        todo = list(self.repos._revmeta_provider._iter_reverse_revmeta_mapping_history(branch, revnum, to_revnum=0, mapping=mapping))
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            for i, (revmeta, mapping) in enumerate(reversed(todo)):
+            for i, (revmeta, hidden, mapping) in enumerate(reversed(todo)):
                 pb.update('generating file id map', i, len(todo))
-                if revmeta.is_hidden(mapping):
+                if hidden:
                     continue
                 self.update_idmap(map, revmeta, mapping)
         finally:
@@ -396,18 +405,16 @@ class CachingFileIdMapStore(object):
 
     def get_map(self, (uuid, branch, revnum), mapping):
         """Make sure the map is up to date until revnum."""
-        if revnum == 0:
-            return self.actual.get_map((uuid, branch, revnum), mapping)
         todo = []
         next_parent_revs = []
 
         # No history -> empty map
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            for revmeta, mapping in self.repos._revmeta_provider._iter_reverse_revmeta_mapping_history(branch, revnum, to_revnum=0, mapping=mapping):
+            for revmeta, hidden, mapping in self.repos._revmeta_provider._iter_reverse_revmeta_mapping_history(branch, revnum, to_revnum=0, mapping=mapping):
                 pb.update("fetching changes for file ids",
                     revnum-revmeta.metarev.revnum, revnum)
-                if revmeta.is_hidden(mapping):
+                if hidden:
                     continue
                 revid = revmeta.get_revision_id(mapping)
                 try:
@@ -422,10 +429,10 @@ class CachingFileIdMapStore(object):
 
         # target revision was present
         if len(todo) == 0:
-            return map
+            return self.actual.get_map((uuid, branch, revnum), mapping)
 
         if len(next_parent_revs) == 0:
-            if mapping.is_branch(""):
+            if mapping.is_branch("") and branch == "":
                 map = DictFileIdMap({
                     u"": (mapping.generate_file_id((uuid, "", 0), u""),
                           NULL_REVISION, None)})
