@@ -16,6 +16,7 @@
 
 """Maps between Subversion and Bazaar semantics."""
 
+from __future__ import absolute_import
 
 import calendar
 from cStringIO import StringIO
@@ -996,7 +997,11 @@ class SubversionMappingRegistry(foreign.VcsMappingRegistry):
     revision_id_bzr_to_foreign = parse_revision_id
 
 
-mapping_registry = SubversionMappingRegistry()
+try:
+    mapping_registry = SubversionMappingRegistry("bzrlib.plugins.svn.mapping",
+        "mapping_registry")
+except TypeError:
+    mapping_registry = SubversionMappingRegistry()
 mapping_registry.register_lazy('v1', 'bzrlib.plugins.svn.mapping2',
                                'BzrSvnMappingv1',
                                'Original bzr-svn mapping format (bzr-svn 0.2.x)')
@@ -1115,11 +1120,11 @@ def revprops_complete(revprops):
             SVN_REVPROP_BZR_HIDDEN in revprops)
 
 
-def get_svn_file_contents(tree, kind, file_id):
+def get_svn_file_contents(tree, kind, file_id, path):
     if kind == "file":
-        return tree.get_file(file_id)
+        return tree.get_file(file_id, path)
     elif kind == "symlink":
-        target = tree.get_symlink_target(file_id)
+        target = tree.get_symlink_target(file_id, path)
         return StringIO("link %s" % target.encode("utf-8"))
     else:
         raise AssertionError
