@@ -15,6 +15,8 @@
 
 """Subversion Meta-Revisions. This is where all the magic happens. """
 
+from __future__ import absolute_import
+
 from subvertpy import (
     properties,
     )
@@ -427,12 +429,6 @@ class BzrMetaRevision(object):
                 return bzr_revno
         return None
 
-    def get_hidden_lhs_ancestors_count(self, mapping):
-        """Get the number of hidden ancestors this revision has."""
-        if not mapping.supports_hidden:
-            return 0
-        return mapping.get_hidden_lhs_ancestors_count(self.get_fileprops())
-
     def get_revno(self, mapping):
         """Determine the Bazaar revision number for this revision.
 
@@ -440,7 +436,8 @@ class BzrMetaRevision(object):
         :return: Bazaar revision number
         """
         extra = 0
-        total_hidden = None
+        total_hidden = 0
+        hidden_seen = 0
         lm = self
         pb = ui.ui_factory.nested_progress_bar()
         try:
@@ -450,15 +447,15 @@ class BzrMetaRevision(object):
                 (mapping, lhs_mapping) = lm.get_appropriate_mappings(mapping)
                 ret = lm.get_distance_to_null(mapping)
                 if ret is not None:
-                    return ret + extra - (total_hidden or 0)
-                if total_hidden is None:
-                    total_hidden = lm.get_hidden_lhs_ancestors_count(mapping)
+                    return ret + extra - total_hidden
+                if lm.is_hidden(mapping):
+                    total_hidden += 1
                 extra += 1
                 lm = lm.get_direct_lhs_parent_revmeta()
                 mapping = lhs_mapping
         finally:
             pb.finished()
-        return extra - (total_hidden or 0)
+        return extra - total_hidden
 
     def get_rhs_parents(self, mapping, parentrevmeta):
         """Determine the right hand side parent ids for this revision.
@@ -947,6 +944,7 @@ class RevisionMetadataProvider(object):
         of a specified path.
 
         """
+        # FIXME: This should rely on mergeinfo data
         todo = []
         processed = set()
         def update_todo(todo, it):
@@ -966,9 +964,11 @@ class RevisionMetadataProvider(object):
             if pb is not None:
                 pb.update("finding rhs ancestors", i, i+len(todo))
             i += 1
-            (revmeta, mapping) = entry
+            (revmeta, hidden, mapping) = entry
             yield entry
             parentrevmeta = revmeta.get_lhs_parent_revmeta(mapping)
+            if hidden:
+                continue
             for rhs_parent_revid in revmeta.get_rhs_parents(mapping, parentrevmeta):
                 try:
                     (rhs_parent_foreign_revid, rhs_parent_mapping) = self.lookup_bzr_revision_id(rhs_parent_revid, foreign_sibling=revmeta.metarev.get_foreign_revid())
@@ -999,6 +999,7 @@ class RevisionMetadataProvider(object):
             except StopIteration:
                 revid = None
                 foreign_sibling = None
+                hidden = False
             else:
                 (mapping, lhs_mapping) = revmeta.get_appropriate_mappings(
                     mapping)
@@ -1007,9 +1008,11 @@ class RevisionMetadataProvider(object):
                         "LHS mapping %r newer than %r" %
                             (lhs_mapping, mapping))
                 if revmeta.is_hidden(mapping):
-                    mapping = lhs_mapping
-                    continue
-                revid = revmeta.get_revision_id(mapping)
+                    revid = None
+                    hidden = True
+                else:
+                    revid = revmeta.get_revision_id(mapping)
+                    hidden = False
                 foreign_sibling = revmeta.metarev.get_foreign_revid()
             if expected_revid is not None and revid != expected_revid:
                 # Need to restart, branch root has changed
@@ -1019,10 +1022,10 @@ class RevisionMetadataProvider(object):
                     expected_revid, foreign_sibling=foreign_sibling)
                 (_, branch_path, revnum) = foreign_revid
                 it = get_iter(branch_path, revnum)
-            elif revid is not None:
+            elif revid is not None or hidden:
                 if not mapping.is_branch_or_tag(revmeta.metarev.branch_path):
                     return
-                yield revmeta, mapping
+                yield revmeta, hidden, mapping
                 expected_revid = revmeta._get_stored_lhs_parent_revid(mapping)
                 mapping = lhs_mapping
             else:
