@@ -16,17 +16,109 @@
 
 """Config tests."""
 
+from bzrlib import bzrdir
 from bzrlib.branch import Branch
 from bzrlib.repository import Repository
-from bzrlib.workingtree import WorkingTree
+from bzrlib.tests import test_config
 from bzrlib.plugins.svn.config import (
     NoSubversionBuildPackageConfig,
     PropertyConfig,
     SubversionBuildPackageConfig,
+    SubversionStore,
+    SvnBranchStack,
+    UUIDMatcher,
     )
 from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
 from bzrlib.plugins.svn.tests import SubversionTestCase
 
+
+def build_subversion_store(test):
+    return SubversionStore()
+
+def build_svn_branch_stack(test):
+    """See bt.test_config.build_backing_branch."""
+    if getattr(test, 'backing_branch', None) is None:
+        # First call, let's build the branch on disk
+        repo = test.make_repository('branch', format='subversion')
+        test.backing_branch = bzrdir.BzrDir.open(repo.base
+                                                 ).create_branch(lossy=False)
+    b = Branch.open('branch')
+    return SvnBranchStack(b.base, b.repository.uuid)
+
+
+class TestReadonlySubversionStore(test_config.TestReadonlyStore):
+
+    def setUp(self):
+        super(TestReadonlySubversionStore, self).setUp()
+        self.get_store = build_subversion_store
+
+
+class TestMutableSubversionStore(test_config.TestMutableStore):
+
+    def setUp(self):
+        super(TestMutableSubversionStore, self).setUp()
+        self.store_id = 'subversion'
+        self.get_store = build_subversion_store
+
+
+class TestUUIDMatcherStoreLoading(test_config.TestSectionMatcher):
+
+    def setUp(self):
+        super(TestUUIDMatcherStoreLoading, self).setUp()
+        self.get_store = build_subversion_store
+        self.matcher = UUIDMatcher
+
+
+class TestUUIDMatching(test_config.TestNameMatcher):
+
+    def setUp(self):
+        super(TestUUIDMatching, self).setUp()
+        self.get_store = build_subversion_store
+        self.matcher = UUIDMatcher
+
+
+class TestConcurrentStoreUpdates(test_config.TestConcurrentStoreUpdates):
+
+    def setUp(self):
+        self.get_stack = build_svn_branch_stack
+        super(TestConcurrentStoreUpdates, self).setUp()
+
+
+class TestConcreteStacks(test_config.TestConcreteStacks):
+
+    def setUp(self):
+        self.get_stack = build_svn_branch_stack
+        super(TestConcreteStacks, self).setUp()
+
+
+class TestStackGet(test_config.TestStackGet):
+
+    def setUp(self):
+        self.get_stack = build_svn_branch_stack
+        super(TestStackGet, self).setUp()
+
+
+class TestStackGetWithConverter(test_config.TestStackGetWithConverter):
+
+    def setUp(self):
+        self.get_stack = build_svn_branch_stack
+        super(TestStackGetWithConverter, self).setUp()
+
+
+class TestStackSet(test_config.TestStackSet):
+
+    def setUp(self):
+        self.get_stack = build_svn_branch_stack
+        super(TestStackSet, self).setUp()
+
+
+class TestStackRemove(test_config.TestStackRemove):
+
+    def setUp(self):
+        self.get_stack = build_svn_branch_stack
+        super(TestStackRemove, self).setUp()
+
+
 class ReposConfigTests(SubversionTestCase):
 
     def setUp(self):
@@ -50,26 +142,6 @@ class ReposConfigTests(SubversionTestCase):
         c.add_location("brainslug")
         self.assertEquals(set(["foobar", "brainslug", self.repos.base.rstrip("/")]), c.get_locations())
 
-    def test_get_branches(self):
-        c = self.config
-        c.set_user_option("branches", "bla;blie")
-        self.assertEquals(["bla", "blie"], c.get_branches())
-
-    def test_branches(self):
-        c = self.config
-        c.set_branches(["bla", "blie"])
-        self.assertEquals(["bla", "blie"], c.get_branches())
-
-    def test_get_tags(self):
-        c = self.config
-        c.set_user_option("tags", "bla;blie")
-        self.assertEquals(["bla", "blie"], c.get_tags())
-
-    def test_tags(self):
-        c = self.config
-        c.set_tags(["bla", "blie"])
-        self.assertEquals(["bla", "blie"], c.get_tags())
-
     def test_get_scheme_none(self):
         c = self.config
         self.assertEquals(None, c.get_branching_scheme())
@@ -116,38 +188,60 @@ class BranchConfigTests(SubversionTestCase):
     def test_has_explicit_nickname(self):
         self.assertEquals(False, self.config.has_explicit_nickname())
 
+class BranchConfigStackTests(SubversionTestCase):
+
+    def setUp(self):
+        super(BranchConfigStackTests, self).setUp()
+        self.config = self.make_svn_branch("d").get_config_stack()
+
     def test_set_option(self):
-        self.config.set_user_option("append_revisions_only", "True")
-        self.assertEquals("True", self.config.get_user_option("append_revisions_only"))
+        self.config.set("append_revisions_only", "True")
+        self.assertEquals(True, self.config.get('append_revisions_only'))
 
     def test_log_strip_trailing_newline(self):
-        self.assertEquals(False, self.config.get_log_strip_trailing_newline())
-        self.config.set_user_option("log-strip-trailing-newline", "True")
-        self.assertEquals(True, self.config.get_log_strip_trailing_newline())
-        self.config.set_user_option("log-strip-trailing-newline", "False")
-        self.assertEquals(False, self.config.get_log_strip_trailing_newline())
+        self.assertEquals(False, self.config.get('log-strip-trailing-newline'))
+        self.config.set("log-strip-trailing-newline", "True")
+        self.assertEquals(True, self.config.get('log-strip-trailing-newline'))
+        self.config.set("log-strip-trailing-newline", "False")
+        self.assertEquals(False, self.config.get('log-strip-trailing-newline'))
 
     def test_override_revprops(self):
-        self.assertEquals(None, self.config.get_override_svn_revprops())
-        self.config.set_user_option("override-svn-revprops", "True")
-        self.assertEquals(["svn:date", "svn:author"], self.config.get_override_svn_revprops())
-        self.config.set_user_option("override-svn-revprops", "False")
-        self.assertEquals([], self.config.get_override_svn_revprops())
-        self.config.set_user_option("override-svn-revprops", ["svn:author", "svn:date"])
-        self.assertEquals(["svn:author", "svn:date"], self.config.get_override_svn_revprops())
-        self.config.set_user_option("override-svn-revprops", ["svn:author"])
-        self.assertEquals(["svn:author"], self.config.get_override_svn_revprops())
-        self.config.set_user_option("override-svn-revprops", ["svn:author=author"])
-        self.assertEquals(["svn:author=author"], self.config.get_override_svn_revprops())
-        self.config.set_user_option("override-svn-revprops", ["svn:author=committer"])
-        self.assertEquals(["svn:author=committer"], self.config.get_override_svn_revprops())
+        self.assertEquals(None, self.config.get('override-svn-revprops'))
+        self.config.set("override-svn-revprops", "True")
+        self.assertEquals(["svn:date", "svn:author"], self.config.get('override-svn-revprops'))
+        self.config.set("override-svn-revprops", "False")
+        self.assertEquals([], self.config.get('override-svn-revprops'))
+        self.config.set("override-svn-revprops", "svn:author,svn:date")
+        self.assertEquals(["svn:author", "svn:date"], self.config.get('override-svn-revprops'))
+        self.config.set("override-svn-revprops", "svn:author")
+        self.assertEquals(["svn:author"], self.config.get('override-svn-revprops'))
+        self.config.set("override-svn-revprops", "svn:author=author")
+        self.assertEquals(["svn:author=author"], self.config.get('override-svn-revprops'))
+        self.config.set("override-svn-revprops", "svn:author=committer")
+        self.assertEquals(["svn:author=committer"], self.config.get('override-svn-revprops'))
 
     def test_get_append_revisions_only(self):
-        self.assertEquals(True, self.config.get_append_revisions_only())
-        self.config.set_user_option("append_revisions_only", "True")
-        self.assertEquals(True, self.config.get_append_revisions_only())
-        self.config.set_user_option("append_revisions_only", "False")
-        self.assertEquals(False, self.config.get_append_revisions_only())
+        self.assertEquals(None, self.config.get('append_revisions_only'))
+        self.config.set("append_revisions_only", "True")
+        self.assertEquals(True, self.config.get('append_revisions_only'))
+        self.config.set("append_revisions_only", "False")
+        self.assertEquals(False, self.config.get('append_revisions_only'))
+
+    def test_get_branches(self):
+        c = self.config
+        self.assertIs(None, c.get('branches'))
+        c.set("branches", "bla;blie")
+        self.assertEquals(["bla", "blie"], c.get('branches'))
+
+    def test_get_tags(self):
+        c = self.config
+        c.set("tags", "bla;blie")
+        self.assertEquals(["bla", "blie"], c.get('tags'))
+
+    def test_tags(self):
+        c = self.config
+        c.set("tags", "bla;blie")
+        self.assertEquals(["bla", "blie"], c.get('tags'))
 
 
 class PropertyConfigTests(SubversionTestCase):
