@@ -83,6 +83,18 @@ from bzrlib.plugins.svn.tests import (
     )
 
 
+def revision_history(branch):
+    branch.lock_read()
+    try:
+        graph = branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(branch.last_revision(),
+            [NULL_REVISION]))
+    finally:
+        branch.unlock()
+    history.reverse()
+    return history
+
+
 class TestDPush(SubversionTestCase):
 
     def setUp(self):
@@ -224,12 +236,15 @@ class TestPush(SubversionTestCase):
         dc = self.commit_editor()
         foo = dc.add_dir("foo")
         foo.add_file("foo/bla").modify("data")
-        dc.close()
+        dc.close() #1
 
         self.svndir = BzrDir.open(self.repos_url)
         os.mkdir("dc")
         self.bzrdir = self.svndir.sprout("dc")
 
+        repo = self.svndir.open_repository()
+        self.revid1 = repo.generate_revision_id(1, "", repo.get_mapping())
+
     def commit_editor(self, message="Test commit"):
         return self.get_commit_editor(self.repos_url, message)
 
@@ -393,7 +408,7 @@ class TestPush(SubversionTestCase):
         wt = self.bzrdir.open_workingtree()
         wt.add('file')
         self.build_tree({'dc/foo/bla': 'data43243242'})
-        revid = wt.commit(message="Commit from Bzr")
+        revid = wt.commit(message="Commit from Bzr") #2
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
@@ -406,7 +421,7 @@ class TestPush(SubversionTestCase):
             self.assertEqual(rtree.get_file_revision(rtree.path2id("file")),
                              revid)
             self.assertEqual(rtree.get_file_revision(rtree.path2id("foo")),
-                             b.revision_history()[1])
+                             self.revid1)
             self.assertEqual(rtree.get_file_revision(rtree.path2id("foo/bla")),
                              revid)
             self.assertEqual(rtree.get_revision_id(), b.last_revision())
@@ -532,7 +547,7 @@ class TestPush(SubversionTestCase):
         revid = wt.commit(message="Commit from Bzr")
 
         b = Branch.open("%s/trunk" % self.repos_url)
-        wt.branch.push(b, stop_revision=wt.branch.revision_history()[-2])
+        wt.branch.push(b, stop_revision=revision_history(wt.branch)[-2])
         mutter('log %r' % self.client_log("%s/trunk" % self.repos_url, 0, 4)[4][0])
         if not b.mapping.can_use_revprops and b.mapping.can_use_fileprops:
             self.assertEquals('M',
@@ -897,9 +912,11 @@ class PushNewBranchTests(SubversionTestCase):
         newbranch = newdir.import_branch(bzrwt.branch)
         self.assertEquals(revid2, newbranch.last_revision())
         mapping = svnrepos.get_mapping()
-        self.assertEquals([
-            svnrepos.generate_revision_id(1, "trunk", mapping)
-            , revid1, revid2], newbranch.revision_history())
+        self.addCleanup(newbranch.lock_read().unlock)
+        graph = newbranch.repository.get_graph()
+        self.assertEquals([revid2, revid1,
+            svnrepos.generate_revision_id(1, "trunk", mapping)],
+            list(graph.iter_lefthand_ancestry(newbranch.last_revision())))
 
     def test_push_overwrite(self):
         repos_url = self.make_svn_repository("a")
@@ -1008,7 +1025,7 @@ class PushNewBranchTests(SubversionTestCase):
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
         def check(b):
-            self.assertEquals([revid1, revid2, revid3], b.revision_history())
+            self.assertEquals([revid1, revid2, revid3], revision_history(b))
             tree = b.repository.revision_tree(revid3)
             self.assertEquals("origid", tree.path2id("registry.moved/c.c"))
             self.assertEquals("newid", tree.path2id("registry.moved/generic.c"))
@@ -1026,23 +1043,22 @@ class PushNewBranchTests(SubversionTestCase):
         revid1 = bzrwt.commit("Initial")
         bzrwt.lock_write()
         try:
-            old_ie = bzrwt.inventory.root.copy()
             new_ie = bzrwt.inventory.root.copy()
             foo_ie = bzrwt.inventory[bzrwt.path2id("foo")].copy()
             new_ie.file_id = "mynewroot"
             foo_ie.parent_id = new_ie.file_id
             bzrwt.apply_inventory_delta([
-                ("", None, old_ie.file_id, None),
+                ("", None, bzrwt.get_root_id(), None),
                 (None, "", new_ie.file_id, new_ie),
                 ("foo", "foo", foo_ie.file_id, foo_ie)])
         finally:
             bzrwt.unlock()
         revid2 = bzrwt.commit(message="Commit from Bzr")
-        return bzrwt, old_ie, new_ie, foo_ie, revid1, revid2
+        return bzrwt, new_ie, foo_ie, revid1, revid2
 
     def test_change_root(self):
         repos_url = self.make_svn_repository("a")
-        bzrwt, old_ie, new_ie, foo_ie, revid1, revid2 = self._create_bzrwt_with_changed_root()
+        bzrwt, new_ie, foo_ie, revid1, revid2 = self._create_bzrwt_with_changed_root()
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
         self.assertChangedPathsEquals({"trunk": ("R", None, -1, NODE_DIR),
@@ -1050,7 +1066,6 @@ class PushNewBranchTests(SubversionTestCase):
             newbranch.repository._revmeta_provider.get_revision("trunk", 2).metarev.paths)
         tree1 = newbranch.repository.revision_tree(revid1)
         tree2 = newbranch.repository.revision_tree(revid2)
-        self.assertEquals(tree1.path2id(""), old_ie.file_id)
         self.assertEquals(tree2.path2id(""), new_ie.file_id)
         self.assertEquals(tree1.path2id("foo"), foo_ie.file_id)
         self.assertEquals(tree2.path2id("foo"), foo_ie.file_id)
@@ -1060,7 +1075,7 @@ class PushNewBranchTests(SubversionTestCase):
 
     def test_change_root_fetch(self):
         repos_url = self.make_svn_repository("a")
-        (bzrwt, old_ie, new_ie, foo_ie, revid1, revid2) = \
+        (bzrwt, new_ie, foo_ie, revid1, revid2) = \
             self._create_bzrwt_with_changed_root()
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
@@ -1080,7 +1095,6 @@ class PushNewBranchTests(SubversionTestCase):
 
         tree1 = newrepos.revision_tree(revid1)
         tree2 = newrepos.revision_tree(revid2)
-        self.assertEquals(tree1.path2id(""), old_ie.file_id)
         self.assertEquals(tree2.path2id(""), new_ie.file_id)
         self.assertEquals(tree1.path2id("foo"), foo_ie.file_id)
         self.assertEquals(tree2.path2id("foo"), foo_ie.file_id)
@@ -1097,7 +1111,7 @@ class PushNewBranchTests(SubversionTestCase):
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
         def check(b):
-            self.assertEquals([revid1, revid2], b.revision_history())
+            self.assertEquals([revid1, revid2], revision_history(b))
             tree = b.repository.revision_tree(revid2)
             self.assertEquals("origid", tree.path2id("registry.moved/generic.c"))
             self.assertEquals("dirid", tree.path2id("registry.moved"))
@@ -1152,7 +1166,7 @@ class PushNewBranchTests(SubversionTestCase):
         trunk.set_append_revisions_only(False)
         trunk.pull(bzrwt.branch)
 
-        self.assertEquals([revid1, revid2, revid3], trunk.revision_history())
+        self.assertEquals([revid1, revid2, revid3], revision_history(trunk))
 
     def test_complex_replace_dir(self):
         repos_url = self.make_svn_repository("a")
@@ -1171,7 +1185,7 @@ class PushNewBranchTests(SubversionTestCase):
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
         self.assertEquals(revid2, newbranch.last_revision())
-        self.assertEquals([revid1, revid2], newbranch.revision_history())
+        self.assertEquals([revid1, revid2], revision_history(newbranch))
 
         os.mkdir("n")
         BzrDir.open(repos_url+"/trunk").sprout("n")
@@ -1251,20 +1265,20 @@ class TestPushTwice(SubversionTestCase):
         bzrdir = svndir.sprout("dc")
         wt = bzrdir.open_workingtree()
         revid = wt.commit(message="Commit from Bzr")
-        expected_history = wt.branch.revision_history()
+        expected_history = revision_history(wt.branch)
 
         svndir1 = BzrDir.open(repos_url+"/branches/a")
         svndir1.import_branch(wt.branch)
-        self.assertEquals(expected_history, svndir1.open_branch().revision_history())
+        self.assertEquals(expected_history, revision_history(svndir1.open_branch()))
 
         svndir2 = BzrDir.open(repos_url+"/branches/b")
         svndir2.import_branch(wt.branch)
-        self.assertEquals(expected_history, svndir2.open_branch().revision_history())
+        self.assertEquals(expected_history, revision_history(svndir2.open_branch()))
 
         svndir1.open_branch().pull(wt.branch)
-        self.assertEquals(expected_history, svndir1.open_branch().revision_history())
+        self.assertEquals(expected_history, revision_history(svndir1.open_branch()))
         svndir2.open_branch().pull(wt.branch)
-        self.assertEquals(expected_history, svndir2.open_branch().revision_history())
+        self.assertEquals(expected_history, revision_history(svndir2.open_branch()))
 
 
 class DetermineBranchPathTests(TestCase):
