@@ -15,6 +15,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Checkouts and working trees (working copies)."""
 
+from __future__ import absolute_import
+
 from collections import (
     defaultdict,
     deque,
@@ -699,6 +701,7 @@ class SvnWorkingTree(SubversionTree, WorkingTree):
         raise NoSuchId(self, file_id)
 
     def _ie_from_entry(self, relpath, entry, parent_id):
+        assert type(parent_id) is str or parent_id is None
         assert type(relpath) == unicode
         (file_id, revid) = self._find_ids(relpath, entry)
         abspath = self.abspath(relpath)
@@ -733,6 +736,12 @@ class SvnWorkingTree(SubversionTree, WorkingTree):
                 ie.executable = self.is_executable(id, relpath)
                 return ie
 
+    def iter_children(self, file_id, path=None):
+        """See Tree.iter_children."""
+        entry = self.iter_entries_by_dir([file_id]).next()[1]
+        for child in getattr(entry, 'children', {}).itervalues():
+            yield child.file_id
+
     def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
         """See WorkingTree.iter_entries_by_dir."""
         if specific_file_ids is not None:
@@ -748,7 +757,7 @@ class SvnWorkingTree(SubversionTree, WorkingTree):
                     entry = self._get_entry(wc, path)
                     if entry.schedule == SCHEDULE_DELETE:
                         continue
-                    ie = self._ie_from_entry(path, entry, parent_id)
+                    ie = self._ie_from_entry(path, entry, parent_id[0])
                     if ie is not None:
                         yield path, ie
             finally:
@@ -998,7 +1007,7 @@ class SvnWorkingTree(SubversionTree, WorkingTree):
             return self.branch.basis_tree()
 
     def list_files(self, include_root=False, from_dir=None, recursive=True):
-        """See `Tree.list_files`."""
+        """See ``Tree.list_files``."""
         # TODO: This doesn't sort the output
         # TODO: This ignores unversioned files at the moment
         if from_dir is None:
@@ -1022,6 +1031,7 @@ class SvnWorkingTree(SubversionTree, WorkingTree):
                     parent_id = fileids[parent_path]
                 except KeyError:
                     parent_id = self.path2id(parent_path)
+                    assert type(parent_id) is str
                     fileids[parent_path] = parent_id
             ie = self._ie_from_entry(relpath, entry, parent_id)
             if ie is not None:
